import { ErrorCodeEnum } from '@/common/enum/ErrorCodeEnum';
import { v4 } from 'uuid';

export interface InternalJsonError {
  uniqueId: string;
  className: string;
  code: number;
  data: Record<string, any>;
  message: string;
  stack: string;
}

/**
 * 所有已知异常的基类，throw 语句抛出的异常必须继承自这个类型
 *
 * 如果需要在业务逻辑中单独处理某种异常，请选择一个最合适的子类并继承它来创建自己的异常类型，然后使用 instanceof 在 catch 语句中捕获它
 */
export abstract class BaseError extends Error {
  private readonly uniqueId = v4();
  private data: Record<string, any> = {};
  private privateData: Record<string, any> = {};

  /**
   * 可选的 API 码，参见 ErrorCodeEnum
   */
  protected getErrorCode(): ErrorCodeEnum {
    return undefined;
  }

  /**
   * 所有异常类的构造器必须为 protected 或 private，通过静态工厂方法选择性开放给外部调用
   */
  protected constructor(
    message: string,
    data?: Record<string, any>,
    privateData?: Record<string, any>,
  ) {
    super(message);
    this.data = data;
    this.privateData = privateData;
  }

  /**
   * 获取自定义数据字段，可重写
   */
  getData(): Record<string, any> {
    return this.data;
  }

  /**
   * 获取私有自定义数据字段，限定内部可见，可重写
   */
  getPrivateData(): Record<string, any> {
    return this.privateData;
  }

  /**
   * 转换为对外显示用字符串
   */
  toString() {
    return JSON.stringify(this.toJson());
  }

  protected setData(data: Record<string, any>) {
    this.data = data;
  }

  protected setPrivateData(data: Record<string, any>) {
    this.privateData = data;
  }

  /**
   * 转换为对外显示用 JSON
   *
   * 不要将此方法用于业务逻辑！
   */
  toJson() {
    return {
      uniqueId: this.uniqueId,
      code: this.getErrorCode(),
      data: this.getData(),
      message: this.message,
    };
  }

  /**
   * 转换为内部调试用字符串
   *
   * 不要将此方法用于业务逻辑！
   */
  toStringInternal() {
    return JSON.stringify(this.toJsonInternal());
  }

  /**
   * 转换为内部调试用 JSON
   *
   * 不要将此方法用于业务逻辑！
   */
  toJsonInternal(): InternalJsonError {
    return {
      uniqueId: this.uniqueId,
      className: this.constructor.name,
      code: this.getErrorCode(),
      data: {
        ...this.getData(),
        ...this.getPrivateData(),
      },
      message: this.message,
      stack: this.stack,
    };
  }
}
