import {
  Controller,
  Post,
  Body,
  Req,
  UseInterceptors,
  UseFilters,
} from '@nestjs/common';
import { AuthService } from './auth.service';
import { LoginDto } from './dto/req/login.dto';
import { IRequest } from '@/interface';
import { LoginRecordDto } from './dto/loginRecord.dto';
import { ApiResponseInterceptor } from '@/common/interceptor/api.response.interceptor';
import { ApiExceptionsFilter } from '@/common/filter/api.exception.filter';
import { RegisterDto } from './dto/req/register.dto';
import { Auth } from '@/common/decorators/auth.decorator';
import { ApiTags, ApiOperation, ApiResponse } from '@nestjs/swagger';
import { LoginResDto } from './dto/res/loginRes.dto';
import { TxAsrResDto } from './dto/res/txAsrRes.dto';
import { plainToClass } from 'class-transformer';

@ApiTags('auth')
@Controller('auth')
@UseInterceptors(ApiResponseInterceptor)
@UseFilters(ApiExceptionsFilter)
export class AuthController {
  constructor(private readonly authService: AuthService) {}

  // 登录接口
  @ApiOperation({ summary: '用户登录' })
  @ApiResponse({ status: 200, description: '登录成功', type: LoginResDto })
  @ApiResponse({ status: 401, description: '认证失败' })
  @Post('login')
  async login(
    @Body() loginDto: LoginDto,
    @Req() req: IRequest,
  ): Promise<LoginResDto> {
    const record: LoginRecordDto = {
      userId: null,
      ip: req.ip,
      lastActiveTime: Math.floor(Date.now() / 1000),
      level: 0,
      platform: 9,
      userAgent: req.userAgent,
    };

    // 调用服务层登录方法
    return await this.authService.login(loginDto, record);
  }

  // 注册接口
  @ApiOperation({ summary: '用户注册' })
  @ApiResponse({ status: 201, description: '注册成功', type: LoginResDto })
  @ApiResponse({ status: 400, description: '请求参数错误' })
  @Post('register')
  async register(@Body() registerDto: RegisterDto): Promise<LoginResDto> {
    return await this.authService.register(registerDto);
  }

  // 重置密码接口
  @ApiOperation({ summary: '用户重置密码' })
  @ApiResponse({ status: 200, description: '重置密码成功', type: LoginResDto })
  @ApiResponse({ status: 400, description: '请求参数错误' })
  @Post('resetPwd')
  async resetPwd(@Body() registerDto: RegisterDto): Promise<LoginResDto> {
    return await this.authService.resetPwd(registerDto);
  }

  // 退出登录接口
  @ApiOperation({ summary: '用户退出登录' })
  @ApiResponse({ status: 200, description: '退出登录成功' })
  @ApiResponse({ status: 401, description: '认证失败' })
  @Auth()
  @Post('logout')
  async logout(@Req() req: IRequest): Promise<void> {
    return await this.authService.logout(req.token, req.decodedToken.exp);
  }

  // 获取ASR token
  @ApiOperation({ summary: '获取ASR token' })
  @ApiResponse({
    status: 200,
    description: '获取ASR token成功',
    type: TxAsrResDto,
  })
  @Auth()
  @Post('getAsrToken')
  async getAsrToken(): Promise<TxAsrResDto> {
    const result = await this.authService.getTxAsrToken();

    if (!result) {
      throw new Error('获取ASR token失败');
    }

    return plainToClass(TxAsrResDto, result, { excludeExtraneousValues: true });
  }
}
