import { CanActivate, ExecutionContext, Injectable } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { config } from '@/config';
import { Request } from 'express';
import { UnauthorizedError } from '@/common/exception/unauthorized/UnauthorizedError';
import { UserService } from '../user/user.service';
import { AuthService } from './auth.service';

@Injectable()
export class AuthGuard implements CanActivate {
  constructor(
    private jwtService: JwtService,
    private authService: AuthService,
    private readonly userService: UserService,
  ) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const request = context.switchToHttp().getRequest();
    const token = this.extractTokenFromHeader(request);
    if (!token) {
      throw UnauthorizedError.default('Token not found');
    }
    try {
      const payload = await this.jwtService.verifyAsync(token, {
        secret: config.jwt.secret,
      });
      // 💡 We're assigning the payload to the request object here
      // so that we can access it in our route handlers
      request['decodedToken'] = payload;
      request['token'] = token;

      const isBlack = await this.authService.isTokenInBlackList(token);

      if (isBlack) {
        throw UnauthorizedError.default('Invalid token');
      }

      const user = await this.userService.getUserById(payload.sub);

      if (!user) {
        throw UnauthorizedError.default('User not found');
      }

      request['user'] = user;
    } catch {
      throw UnauthorizedError.default('Invalid token');
    }
    return true;
  }

  private extractTokenFromHeader(request: Request): string | undefined {
    const [type, token] = request.headers.authorization?.split(' ') ?? [];
    return type === 'Bearer' ? token : undefined;
  }
}
