import {
  Body,
  Controller,
  Get,
  Post,
  Query,
  Res,
  UploadedFile,
  UseInterceptors,
} from '@nestjs/common';
import { TranslateService } from './translate.service';
import { Auth } from '@/common/decorators/auth.decorator';
import { ApiTags, ApiOperation, ApiResponse } from '@nestjs/swagger';
import { TranslateReqDto } from './dto/req/translateReq.dto';
import { TranslateResDto } from './dto/res/translateRes.dto';
import { FileInterceptor } from '@nestjs/platform-express';
import { TranslateImageReqDto } from './dto/req/translateImageReq.dto';
import { TranslateProgressReqDto } from './dto/req/translateProgressReq.dto';
import { ApiResponseInterceptor } from '@/common/interceptor/api.response.interceptor';
import { Response } from 'express';
import { TextToSpeechReqDto } from './dto/req/textToSpeechReq.dto';
import { TTS_VOICE_LIST } from '@/common/utils/constants';
import { BadRequestError } from '@/common/exception/badRequest/BadRequestError';
import { config } from '@/config';

@ApiTags('translate')
@UseInterceptors(ApiResponseInterceptor)
@Controller('translate')
export class TranslateController {
  constructor(private readonly translateService: TranslateService) {}

  // 获取翻译支持的语言
  @Get('languages')
  @ApiOperation({ summary: '获取翻译支持的语言' })
  @ApiResponse({
    status: 200,
    description: '成功返回支持的语言列表',
    example: {
      code: 200,
      message: 'success',
      data: {
        source: [{ key: 'AR', language: 'Arabic', name: '阿拉伯语' }],
        target: [{ key: 'AR', language: 'Arabic', name: '阿拉伯语' }],
      },
    },
  })
  @Auth()
  async getLanguages(): Promise<{ source: object[]; target: object[] }> {
    return await this.translateService.getLanguages();
  }

  // 翻译文本
  @Post('text')
  @ApiOperation({ summary: '翻译文本' })
  @ApiResponse({
    status: 200,
    description: '成功返回翻译结果',
    example: {
      code: 200,
      message: 'success',
      data: {
        source: 'ZH',
        text: 'weekly',
      },
    },
  })
  @Auth()
  async translateText(
    @Body() translateDto: TranslateReqDto,
  ): Promise<TranslateResDto> {
    return await this.translateService.translateText(translateDto);
  }

  // 翻译图片支持的语言
  @Get('image/languages')
  @ApiOperation({ summary: '获取翻译图片支持的语言' })
  @ApiResponse({
    status: 200,
    description: '成功返回支持的语言列表',
    example: {
      code: 200,
      message: 'success',
      data: {
        source: [{ key: 'en', language: 'English', name: '英语' }],
        target: [{ key: 'zh', language: 'Chinese', name: '中文' }],
      },
    },
  })
  @Auth()
  async getImageLanguages(): Promise<{
    source: object[];
    target: object[];
  }> {
    return await this.translateService.getFanYiGouLanguages();
  }

  // 翻译图片
  @Post('image')
  @ApiOperation({ summary: '翻译图片' })
  @ApiResponse({
    status: 200,
    description: '成功返回翻译结果',
    example: {
      code: 200,
      message: 'success',
      data: {
        taskId: 709394,
      },
    },
  })
  @Auth()
  @UseInterceptors(
    FileInterceptor('file', {
      limits: { fileSize: 4 * 1024 * 1024 },
    }),
  )
  async translateImage(
    @UploadedFile() file,
    @Body() dto: TranslateImageReqDto,
  ): Promise<any> {
    return await this.translateService.translateImage(file, dto);
  }

  // 获取翻译进度
  @Get('progress')
  @ApiOperation({ summary: '获取翻译进度' })
  @ApiResponse({
    status: 200,
    description: '成功返回翻译进度',
    example: {
      code: 200,
      message: 'success',
      data: {
        tid: 709412,
        createTime: '2025-01-15 16:54:08',
        updateTime: '',
        status: 1,
        title: '20250111-165032',
        pageCount: 1,
        percent: '0.00',
        fromLan: '英语',
        detectLan: '',
        toLan: '中文',
        docType: 'jpeg',
        msg: '进行中',
        originalCharCount: 0,
      },
    },
  })
  @Auth()
  async getProgress(@Query() dto: TranslateProgressReqDto): Promise<any> {
    return await this.translateService.getProgress(dto.taskId);
  }

  // 下载翻译结果
  @Get('downloadImage')
  @ApiOperation({ summary: '下载翻译结果' })
  @ApiResponse({
    status: 200,
    description: '成功返回翻译结果',
  })
  @Auth()
  async getResult(
    @Query() dto: TranslateProgressReqDto,
    @Res() res: Response,
  ): Promise<any> {
    return await this.translateService.downloadImage(dto.taskId, res);
  }

  // text合成语音
  @Post('textToSpeech')
  @ApiOperation({ summary: 'text合成语音' })
  @ApiResponse({
    status: 200,
    description: '成功返回合成语音文件',
    example: {
      code: 200,
      message: 'success',
      data: {
        url: 'http://xxx.com/xxx.mp3',
      },
    },
  })
  @Auth()
  async textToSpeech(@Body() dto: TextToSpeechReqDto): Promise<any> {
    const voice = TTS_VOICE_LIST.find((item) => item.uuid === dto.speaker);

    if (!voice) {
      throw BadRequestError.default('音色不存在');
    }

    const result = await this.translateService.tts({
      ...dto,
      speaker: voice.speaker,
      provider: voice.provider,
    });

    return `${config.obs.endpoint}/${result.filename}`;
  }

  // 获取语音合成支持的语音列表
  @Get('voiceList')
  @ApiOperation({ summary: '获取语音合成支持的语音列表' })
  @ApiResponse({
    status: 200,
    description: '成功返回支持的语音列表',
    example: {
      code: 200,
      message: 'success',
      data: [
        {
          uuid: '3e7a6cb0-3bae-4af9-bb7d-abc9f1c5ebe6',
          speaker: 'xiaoyan',
          name: '小燕',
          provider: 9,
        },
      ],
    },
  })
  @Auth()
  async getVoiceList(): Promise<any> {
    return TTS_VOICE_LIST;
  }
}
