import { UserEntity } from '@/entities/user.entity';
import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { RegisterDto } from '../auth/dto/req/register.dto';
import { v4 as uuid } from 'uuid';
import * as sha1 from 'sha1';
import { UserInfoEntity } from '@/entities/userInfo.entity';
import { UserBaseDto } from '../auth/dto/userBase.dto';
import { plainToClass } from 'class-transformer';

@Injectable()
export class UserService {
  constructor(
    @InjectRepository(UserEntity)
    private readonly userRepository: Repository<UserEntity>,
    @InjectRepository(UserInfoEntity)
    private readonly userInfoRepository: Repository<UserInfoEntity>,
  ) {}

  // 用户是否存在
  async existByPhone(phone: string): Promise<boolean> {
    const user = await this.userRepository.findOne({ where: { phone } });
    return !!user;
  }

  // 查找最大用户id
  async findMaxUserId(): Promise<number | undefined> {
    const result = await this.userRepository
      .createQueryBuilder('user')
      .select('MAX(user.id)', 'max')
      .getRawOne();

    return result ? result.max : undefined;
  }

  // 创建用户
  async createUser({ phone, password }: RegisterDto): Promise<UserBaseDto> {
    // 注册
    const userEntity = new UserEntity();
    userEntity.phone = phone;
    userEntity.pwd = sha1(password);
    // 注册类型 0: 手机号注册
    userEntity.regType = 0;
    userEntity.state = 0;
    // 注册来源 18: 翻译机
    userEntity.regSource = 18;
    userEntity.regtime = Math.floor(Date.now() / 1000);
    userEntity.uid = uuid();

    const maxId = await this.findMaxUserId();
    // userEntity.acc = (maxId + 100000000 + Math.random() * 100).toString();
    userEntity.acc = `${maxId + 100000000}${Math.floor(Math.random() * 100)}`;
    // 保存用户
    const user = await this.userRepository.save(userEntity);

    // 创建用户信息
    const userInfo = await this.userInfoRepository.save({
      userId: userEntity.id,
      nickname: `InnAIO-${user.id * 11}`,
    });

    // 转换并返回用户信息
    return plainToClass(
      UserBaseDto,
      { ...userInfo, ...user },
      { excludeExtraneousValues: true },
    );
  }

  // 重置密码
  async updatePwd({ phone, password }: RegisterDto): Promise<UserEntity> {
    const user = await this.userRepository.findOne({ where: { phone } });

    if (user) {
      user.pwd = sha1(password);
      await this.userRepository.save(user);
    }

    return user;
  }

  async getUserById(id: number): Promise<UserEntity> {
    return await this.userRepository.findOneBy({ id });
  }

  async getUserByPhone(phone: string): Promise<UserEntity> {
    return await this.userRepository.findOneBy({ phone });
  }

  async update(user: UserEntity): Promise<void> {
    await this.userRepository.update(user.id, user);
  }
}
